const { Given, When, Then, After, AfterAll, setDefaultTimeout} = require('@cucumber/cucumber');
const { By, Key, Select } = require('selenium-webdriver');

const { getDriver } = require('../../src/driver');
const { expect } = require('chai');

// setup
setDefaultTimeout(30 * 1000);
const driver = getDriver()
driver.manage().window().maximize()

const PUBLIK_IS_DEV_INSTANCE = (process.env.PUBLIK_IS_DEV_INSTANCE || 'false') === 'true'
const PUBLIK_URL = PUBLIK_IS_DEV_INSTANCE ? 'https://{}.dev.publik.love' : (process.env.PUBLIK_URL || 'https://{}.dev.publik.love')
const PUBLIK_PORTAIL_USAGER = PUBLIK_IS_DEV_INSTANCE ? 'combo' : process.env.PUBLIK_PORTAIL_USAGER
const PUBLIK_PORTAIL_AGENT = PUBLIK_IS_DEV_INSTANCE ? 'agent-combo' : process.env.PUBLIK_PORTAIL_AGENT
const PUBLIK_DEMARCHES = PUBLIK_IS_DEV_INSTANCE ? 'wcs' : process.env.PUBLIK_PORTAIL_AGENT
const PAGES = {
  'Portail Usager': PUBLIK_URL.replace('{}', PUBLIK_PORTAIL_USAGER || 'portail'),
  'Portail Agent': PUBLIK_URL.replace('{}', PUBLIK_PORTAIL_AGENT || 'agents'),
  'Portail Agent - Fiches': PUBLIK_URL.replace('{}', PUBLIK_DEMARCHES || 'demarches') + '/backoffice/data',
}
console.log('Available pages:', PAGES)
console.log('Launching tests...')

function sleep(time) {
  return new Promise(resolve => setTimeout(resolve, time));
} 

function getUrlFromPage(pageName) {
  return PAGES[pageName]
}

async function getElementByText(driver, text) {
  return await driver.findElement(By.xpath(`//*[contains(text(), "${text}")]`));
}

async function getCurrentUser(driver) {
  let usernameLink
  try {
    usernameLink = await driver.findElement(By.className('connected-user'));
  } catch (e) {
    if(e.name === 'NoSuchElementError') {
      // maybe we're on the agent portal, try with other selector
      usernameLink = await driver.findElement(By.className('ui-name'));
    } else {
      throw e
    }
  }
  let username = await usernameLink.getText()
  return username
}

async function connect (driver, role) {
  let homeUrl = getUrlFromPage('Portail Agent')
  let username = process.env[`PUBLIK_${role}_USERNAME`] || role
  let password = process.env[`PUBLIK_${role}_PASSWORD`] || 'password'
  await driver.get(homeUrl)
  await (await driver.findElement(By.name('username'))).sendKeys(username)
  await (await driver.findElement(By.name('password'))).sendKeys(password, Key.RETURN)
  expect(await getCurrentUser(driver)).to.not.be.empty
}

async function disconnect(driver) {
  let disconnectLink
  disconnectLink = await driver.findElement(By.className('ui-logout'));
  disconnectLink = await disconnectLink.findElement(By.xpath('./a'))
  await disconnectLink.sendKeys(Key.RETURN)
  await driver.manage().deleteAllCookies()
  await driver.navigate().refresh()
}

async function getFieldByNameOrLabel(driver, nameOrLabel) {
  try {
    // easiest case, there is a field with the same name on the page
    return await driver.findElement(By.name(nameOrLabel));
  } catch (e) {
    if(e.name === 'NoSuchElementError') {
      // We try to find a <label> tag with the corresponding 
      // content
      let expression = `//label[text()="${nameOrLabel}"]`
      let label = await driver.findElement(By.xpath(expression));
      let fieldId = await label.getAttribute("for")
      return await driver.findElement(By.id(fieldId));
    } else {
      throw e
    }
  }
}
Given("je suis sur la page {string}", async function (pageName) {
  let url = getUrlFromPage(pageName)
  await driver.get(url);
});
Given("je visite la page {string}", async function (pageName) {
  let url = getUrlFromPage(pageName)
  await driver.get(url);
});

Given("je visite la page {string} sur {string}", async function (path, site) {
  let siteUrl = getUrlFromPage(site)
  await driver.get(siteUrl + path);
});

When('je saisis {string} dans le champ {string}', async function (text, field) {
  let element = await getFieldByNameOrLabel(driver, field)
  if (await element.getAttribute("data-godo-schema")) {
    // this is a godo editor, we need to target the contenteditable
    // div, not the hidden textarea
    element = await element.findElement(By.xpath('./following-sibling::*'));
    element = await element.findElement(By.xpath('.//*[@contenteditable="true"]'));
  }
  await element.sendKeys(text)
});
When('je sélectionne {string} dans le champ {string}', async function (value, field) {
  let element = await getFieldByNameOrLabel(driver, field)
  if (await element.getAttribute("class")  === 'select2-search__field') {
    // This is a select2 multiple input with autocomplete, we cannot rely on
    // built-in browser mechanisms to select a value
    // so we use the autocomplete feature of the input to type the value
    // and select it with Enter
    await input.sendKeys(value, Key.RETURN)
  } else {
    let select = new Select(element)
    await select.selectByVisibleText(value)
  }
});
// When('je visite {string}', async function (url) {
//   if (url.startsWith('/')) {
//     let currentUrl = new URL(await driver.getCurrentUrl())
//     url = `${currentUrl.protocol}//${currentUrl.hostname}${url}`
//   }
//   return await driver.navigate().to(url)
// });

When('je valide (le formulaire)', async function () {
  let currentField = await driver.switchTo().activeElement();
  await currentField.sendKeys(Key.RETURN)
});


Then('je dois être connecté(e) en tant que {string}', async function (expectedUsername) {
  expect(await getCurrentUser(driver)).to.equal(expectedUsername)
});

When('je clique sur {string}', async function (text) {
  let element = await getElementByText(driver, text)
  await element.sendKeys(Key.RETURN);
});

When('j\'attends {float} seconde(s)', async function (duration) {
  await sleep(duration * 1000)
});

When('je clique sur la ligne {int} du tableau de listing', async function (i) {
  let link = await driver.findElement(By.xpath(`//table[@id="listing"]//tr[${i}]//a`));
  await link.click()
});


Then('je ne dois pas être connecté(e)', async function () {
  await driver.findElement(By.className('login-link'));
});

Then('la page doit afficher {string}', async function (text) {
  let element = await driver.findElement(By.xpath(`//*[contains(text(), "${text}")]`));
});

Then('je dois être redirigé vers la page de la fiche', async function () {
  let currentUrl = new URL(await driver.getCurrentUrl())
  let path = currentUrl.pathname
  expect(path.match(/\/backoffice\/data\/fiche\/(\d+)\//)).to.not.be.null
});

Then('la fiche doit être dans le statut {string}', async function (status) {
  let element = await driver.findElement(By.className('current-status'));
  let match = (await element.getText()).endsWith(status)
  expect(match).to.be.true
});

Given("je suis connecté(e) en tant que {string}", async function (role) {
  await connect(driver, role)
});

Given("je me reconnecte en tant que {string}", async function (role) {
  await disconnect(driver)
  await connect(driver, role)
});

After(async function(){
  await sleep(5000)
});

AfterAll(async function(){
  await driver.quit();
});